
(* Written by Jan Cada *)

program herci(input,output);

const
  MAX_K             = 8192;  {maximalni stupen grafu}
  MAX_N             = 8192; {maximalni pocet bodu v grafu}
  MAX_HERCU_ROLI    = 512; {max(maximalni pocet hercu, maximalni pocet roli)}
  MAX_NAME_LEN = 40; {maximalni delka jmena herce resp. role}

type
  HranaT = record
    v : integer; {koncovy bod hrany, pocatecni bod hrany je dan bodem,
                                         u ktereho je tato hrana definovana}
    t : boolean; {tok hranou}
  end;

  HranyPT = ^hranyT;
  HranyT = array[1 .. 1] of hranaT;

  BodT = record           {pomoci promennych tohoto typu je reprezentovan
                           zadany graf g}
    pocHran :  integer;   {pocet hran vedoucich z tohoto bodu}
    hranyP : HranyPT;     {ukazatel na pole hran vedoucich z tohoto bodu}
    cast : boolean;       {do ktere casti v bipart. grafu bod patri}
  end;

  Hrana1T = record
    v : integer;          {koncovy bod hrany}
    tokP : ^boolean;      {ukazatel na tok odpovidajici hranou v grafu g}
  end;

  Hrany1PT = ^Hrany1T;
  Hrany1T = array[1 .. 1] of Hrana1T;

  Bod1T = record          {pomoci promennych tohoto typu je vytvorena
                           prirustkova sit}
    pocHran : integer;    {pocet hran vedoucich z tohoto bodu}
    hranyP : Hrany1PT;    {ukazatel na pole hran vedoucich z tohoto bodu}
    vzd : integer;        {vzdalenost bodu od zdroje v prirustkove siti}
  end;

  Bod2T = record          {pomoci promennych tohoto typu je vytvorena
                           vrstvena sit}
    pocHran : integer;    {pocet hran vedoucich z tohoto bodu}
    hranyP : Hrany1PT;    {ukazatel na pole hran vedoucich z tohoto bodu}
  end;

  PolozkaCestyT = record  {do techto polozek se uklada dosavadni cesta
                           pri prohledavani vrstvene site}
    bod : integer;
    hrana : integer;
  end;

  ValsT = array[0 .. 0] of integer;
  ValsPT = ^ValsT;

  FrontaC = record 
    n, maxN, prvni : integer;
    xP : ValsPT;    
  end;
    
var
{polozky 1 .. n v nasledujicich polich odpovidaji bodum 1 .. n
                                                       bipartitniho grafu,
{polozka 0 predstavuje pridany zdroj, polozka n+1 pridany spotrebic}
  g : array[0 .. MAX_N + 1] of BodT; {graf G}
  s1 : array[0 .. MAX_N + 1] of Bod1T; {prirustkova sit}
  s2 : array[0 .. MAX_N + 1] of Bod2T; {vrstvena sit}
  n : integer; {pocet bodu}
  fr : FrontaC;
  cesta : array[1 .. MAX_N + 1] of PolozkaCestyT;  {pro poznaceni aktualni
                                       cesty pri prohledavani vrstvene site}
  konec : boolean;

procedure FrontaC_init(var f:FrontaC; aMaxN : integer);
begin
 with f do 
 begin
  maxN := aMaxN;
  n := 0;
  prvni := 0;
  getMem(xP, maxN * sizeof(integer));
 end;
end;

procedure FrontaC_done( var f:FrontaC);
begin
 with f do 
   freeMem(xP, maxN * sizeof(integer));
end;

function FrontaC_jePrazdna( var f:FrontaC) : boolean;
begin
 with f do 
  FrontaC_jePrazdna := n = 0;
end;

function FrontaC_vloz( var f:FrontaC; x : integer) : boolean;
begin
 with f do
 begin
  if n < maxN then begin
    xP^[(prvni + n) mod maxN] := x;
    n := n + 1;
    FrontaC_vloz := TRUE;
  end else FrontaC_vloz := FALSE;
 end; 
end;

function FrontaC_vyber( var f:FrontaC; var x : integer) : boolean;
begin
 with f do 
 begin
  if n > 0 then begin
    x := xP^[prvni];
    prvni := (prvni + 1) mod maxN;
    n := n - 1;
    FrontaC_vyber := TRUE;
  end else FrontaC_vyber := FALSE;
 end; 
end;

type TJmeno  = array[1..MAX_NAME_LEN+1] of char;
     TSeznam = array[1..MAX_HERCU_ROLI] of TJmeno; 

var jmena_hercu : TSeznam;
    jmena_roli  : TSeznam;
    hercu, roli, nem_hercu : integer;
    je_nepritomen : array[1..MAX_HERCU_ROLI] of boolean;
    her_x, rol_x : array[1..MAX_N] of integer;
    hran_x, i_x: integer;
    
procedure nacteniJmena(var jmeno : TJmeno );
var  i: integer;
begin
 i:=1;
 repeat
  read(jmeno[i]);
 until((jmeno[i] in ['A'..'Z']) or (jmeno[i] in ['a'..'z']));
 repeat
  i:= i + 1; 
  read(jmeno[i]);
 until not ((jmeno[i] in ['A'..'Z']) or (jmeno[i] in ['a'..'z']));
 while( i <= MAX_NAME_LEN) do begin jmeno[i]:=chr(0); i:=i+1; end;
end;

function hledej( const jmeno : TJmeno; const seznam : TSeznam; delka : integer) : integer;
var i,j    : integer;
    stejne : boolean;
begin
 i := 0;
 repeat
  j := 0;
  i := i + 1;
  repeat
   j := j + 1;
   stejne := jmeno[j] = seznam[i,j]; 
  until( (not stejne) or (Ord(jmeno[j])=0));  
 until (stejne or (i >= delka));
 if( not stejne ) then i := 0;
 hledej := i;
end;

 
procedure nacteniVstupuAInicializace;
var
  i, pom : integer;
  u, v : integer;
  konec : boolean;
  pomJmeno : TJmeno;
  pomRoli : integer;
  
begin

  readln(hercu, roli, nem_hercu);
  n := hercu + roli;   {nacteni poctu vrcholu}
  for i:=1 to hercu do 
  begin
   nacteniJmena(jmena_hercu[i]);
   je_nepritomen[i] := false;
  end;

  for i:=1 to roli do 
  begin
   nacteniJmena(jmena_roli[i]);
  end;

  hran_x := 0;
  for i:=1 to hercu do 
  begin
   nacteniJmena(pomJmeno); {jmeno herce }
   i_x := hledej( pomJmeno, jmena_hercu, hercu);
    read(pomRoli);
   for pom := 1 to pomRoli do
   begin
    nacteniJmena(pomJmeno); { jmeno role }
    hran_x := hran_x + 1;
    her_x[hran_x] := i_x;
   rol_x[hran_x] := hledej(pomJmeno, jmena_roli, roli)
   end; 
  end;


  for i:=1 to nem_hercu do 
  begin
   nacteniJmena(pomJmeno);
   pom := hledej( pomJmeno, jmena_hercu, hercu);
   if(pom=0) then writeln ('Herec nenalezen')
   else je_nepritomen[pom] := true;
  end;

  for i := 1 to n do   {oznaceni bodu, ktere patri do stejne casti grafu (do roli)}
   g[i].cast := i > hercu;

  getMem(g[0].hranyP, n * sizeof(hranaT));
  g[0].pocHran := 0;
  for i := 1 to n do
    if g[i].cast then begin
      getMem(g[i].hranyP, sizeof(hranaT));
      g[i].hranyP^[1].v := n + 1;  {pridani hrany do uzlu n + 1 (spotrebice)}
      g[i].hranyP^[1].t := FALSE;
      g[i].pocHran := 1;
    end else begin
      getMem(g[i].hranyP, MAX_K * sizeof(hranaT));
      g[i].pocHran := 0;
      g[0].pocHran := g[0].pocHran + 1;
      g[0].hranyP^[g[0].pocHran].v := i;  {pridani hrany z uzlu 0 (zdroje)}
      g[0].hranyP^[g[0].pocHran].t := FALSE;
    end;
  g[n + 1].pocHran := 0;

  for i:=1 to hran_x do 
  begin
    if(not je_nepritomen[her_x[i]]) then 
    begin
      u := her_x[i]; v :=rol_x[i] + hercu;
      if g[u].cast xor g[v].cast then begin
        if (g[u].cast) then begin
          i := u; u := v; v := i;
        end;
        g[u].pocHran := g[u].pocHran + 1;
        i := g[u].pocHran;
        g[u].hranyP^[i].v := v;
        g[u].hranyP^[i].t := FALSE;
      end else begin
        writeln('Graf neni bipartitni kvuli hrane (', u,',',v,')');
        halt(1);
      end
    end;
  end;

  {inicializace promennych tvorici prirustkovou sit}
  getmem(s1[0].hranyP, n * sizeof(hranaT));
  for i := 1 to n do getmem(s1[i].hranyP, (MAX_K + 1) * sizeof(hranaT));
  getmem(s1[n+1].hranyP, n * sizeof(hranaT));

  {inicializace promennych tvorici vrstvenou sit}
  getmem(s2[0].hranyP, n * sizeof(hranaT));
  for i := 1 to n do getmem(s2[i].hranyP, (MAX_K + 1) * sizeof(hranaT));
  getmem(s2[n+1].hranyP, n * sizeof(hranaT));

  {vytvoreni fronty}
  FrontaC_init(fr, n + 2);
end;

procedure vytvoreniPrirustkoveSite;
var
  i, j, u, v : integer;
begin
  {vytvoreni prirustkove site}
  for i := 0 to n + 1 do s1[i].pocHran := 0;
  for i := 0 to n do
    for j := 1 to g[i].pocHran do begin
      if g[i].hranyP^[j].t then begin
        u := g[i].hranyP^[j].v;      {pridani opacne orientovane hrany}
        v := i;
      end else begin
        u := i;                      {pridani puvodne orientovane hrany}
        v := g[i].hranyP^[j].v;
      end;
      s1[u].pocHran := s1[u].pocHran + 1;
      s1[u].hranyP^[s1[u].pocHran].v := v;
      s1[u].hranyP^[s1[u].pocHran].tokP := @g[i].hranyP^[j].t;
    end;
end;

function vytvoreniVrstveneSite : boolean;
var
  i, j, u, pom, vzd : integer;
  pom1 : Hrana1T;
begin
  {urceni vzdalenosti kazdeho bodu od zdroje}
  for i := 0 to n + 1 do s1[i].vzd := -1;
  i := 1;
  FrontaC_vloz(fr, 0); s1[0].vzd := 0;
  FrontaC_vloz(fr, -1);    {-1 oddeluje ve fronte jednotlive vrstvy}
  while not FrontaC_jePrazdna(fr) do begin
    FrontaC_vyber(fr, u);
    if (u >= 0) then begin
      for j := 1 to s1[u].pocHran do begin
        pom := s1[u].hranyP^[j].v;
        if s1[pom].vzd < 0 then begin
          s1[pom].vzd := i;
          FrontaC_vloz(fr,pom);
        end;
      end
    end else begin
      i := i + 1;
      if not FrontaC_jePrazdna(fr) then FrontaC_vloz( fr, -1);
    end
  end;
  if s1[n + 1].vzd > 0 then begin {test, zda je spotrebic dosazitelny}
    {vytvoreni vrstvene site}
    for i := 0 to n + 1 do s2[i].pocHran := 0;
    for i := 0 to n do
      for j := 1 to s1[i].pocHran do begin
        pom1 := s1[i].hranyP^[j];
        if (s1[pom1.v].vzd - s1[i].vzd) = 1 then begin
          {pridani hrany do vrstvene site - pridaji se jen ty, ktere vedou
           z dane vrstvy do vrstvy nasledujici}
          s2[i].pocHran := s2[i].pocHran + 1;
          s2[i].hranyP^[s2[i].pocHran] := pom1;
        end;
      end;
    vytvoreniVrstveneSite := TRUE;
  end else vytvoreniVrstveneSite := FALSE;
end;

{pomocna procedure volana z procedury hledaniCest}
procedure upravTok(bod, hrana : integer);
begin
  s2[bod].hranyP^[hrana].tokP^ := not s2[bod].hranyP^[hrana].tokP^;
end;

{pomocna procedure volana z procedury hledaniCest}
procedure odeberHranu(bod, hrana : integer);
begin
  s2[bod].hranyP^[hrana] := s2[bod].hranyP^[s2[bod].pocHran];
  s2[bod].pocHran := s2[bod].pocHran - 1;
end;

procedure hledaniCest;
var
  i, j  : integer;
begin
  cesta[1].bod := 0;    {hledani zacina od bodu 0 - zdroje}
  cesta[1].hrana := 1;
  i := 1;
  while cesta[1].hrana <= s2[0].pocHran do begin
    if cesta[i].hrana <= s2[cesta[i].bod].pocHran then begin
      {prechod po hrane do dalsi vrstvy}
      cesta[i + 1].hrana := 1;
      cesta[i + 1].bod := s2[cesta[i].bod].hranyP^[cesta[i].hrana].v;
      i := i + 1;
      if cesta[i].bod = n + 1 then begin {test, zda bylo dosazeno
                                                                spotrebice}
        for j := i - 1 downto 1 do begin
          upravTok(cesta[j].bod, cesta[j].hrana);  {upraveni toku v grafu g}
          odeberHranu(cesta[j].bod, cesta[j].hrana); {vyjmuti hrany
                                                           z vrstvene site}
        end;
        i := 1;
      end;
    end else begin
      i := i - 1; {neexistuje uz hrana do dalsi vrstvy, navrat zpet}
      cesta[i].hrana := cesta[i].hrana + 1;
    end
  end
end;

procedure vysledek; {vypsani vysledku}
var
  i, j, k : integer;
begin
{  writeln('Hrany patrici do parovani:'); }
  k:=0;
  for i := 1 to n do
    for j := 1 to g[i].pocHran do
      if g[i].hranyP^[j].t and (g[i].hranyP^[j].v <= n)
                                  then  k:=k+1;
  if(k=roli) then writeln('Premiera bude!') else writeln('Zatraceni demonstranti!');
end;

procedure ukonceni; {uvolneni pameti}
var
  i : integer;
begin
  FrontaC_done(fr);
  freemem(s2[n+1].hranyP, n * sizeof(hranaT));
  for i := 1 to n do freemem(s2[i].hranyP, (MAX_K + 1) * sizeof(hranaT));
  freemem(s2[0].hranyP, n * sizeof(hranaT));

  freemem(s1[n+1].hranyP, n * sizeof(hranaT));
  for i := 1 to n do freemem(s1[i].hranyP, (MAX_K + 1) * sizeof(hranaT));
  freemem(s1[0].hranyP, n * sizeof(hranaT));

  for i := 1 to n do
    if g[i].cast then freeMem(g[i].hranyP, sizeof(hranaT))
    else freeMem(g[i].hranyP, MAX_K * sizeof(hranaT));

  freeMem(g[0].hranyP, n * sizeof(hranaT));
end;

var 
     uloha,uloh : integer;

begin
  readln(uloh);
  for uloha:=1 to uloh do 
  begin
   nacteniVstupuAInicializace;
   repeat
    vytvoreniPrirustkoveSite;
    konec := not vytvoreniVrstveneSite;
    if not konec then hledaniCest;
   until konec;
   vysledek;
   ukonceni; 
  end; 
end.
